/*
   Initialization functions
   --------------------------------------------------------------------
   Linberto

   Copyright (C) 1999-2001 Diego Javier Grigna <diego@grigna.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "common.h"
#include "fonts.h"

/* Brightness */
int bright;

/* Maximum number of stages */
int max_stages;

/* The file name we opened */
char *openedfile;

/* Number of pixels for the next menu option */
/* See draw_bar(....)                        */
int menu_spacing;
/* This function is called if it's not NULL in loopbar() */
/* It's needed if we need more fx in the menu            */
void (* menu_function)( int);

static void init_load_config( void);
static void init_set_fonts( void);

void init_svgalib( void)
{
 int mode;

 mode = G320x200x256;

 if( vga_init() != 0) {
     fprintf( stderr, _("\nvga_init: Can't initialize svgalib library.\n\n"));
     exit( 1);
 }

 if( !vga_hasmode( mode)) {
     fprintf( stderr, _("\nvga_hasmode: Sorry mode 320x200x256 isn't supported by your card.\n\n"));
     exit( 1);
 }

 /*
  * I check vga_functions() return values because I had lot's of problems
  * using modes > 320x200 with old svgalibs (< 1.2.x). So now I always
  * check their return values to prevent system lockups.
  */

 if( vga_setmode( mode) != 0) {
     fprintf( stderr, _("\nvga_setmode: Can't set mode 320x200x256.\n\n"));
     exit( 1);
 }

 gl_setcontextvga( mode);
 physicalscreen = gl_allocatecontext();
 gl_getcontext( physicalscreen);

 gl_setcontextvgavirtual( mode);
 backscreen = gl_allocatecontext();
 gl_getcontext( backscreen);

 gl_setcontextvgavirtual( mode);
 spritescreen = gl_allocatecontext();
 gl_getcontext( spritescreen);

 gl_setcontext( physicalscreen);

 if( keyboard_init()) {
     fprintf( stderr, _("\nkeyboard_init: Can't initialize keyboard.\n\n"));
     exit_game( 1);
 }

 keyboard_seteventhandler( key_handler);

 /* Init the fonts, for text writing */
 init_set_fonts();

 printf( "\n\n");
 printf( _("Linberto - version %s\n"), LINBERTO_VERSION);
 printf( _("Copyright (C) 1999-2001 Diego Javier Grigna <diego@grigna.com>\n"));
 printf( _("Linberto is covered by the GNU General Public License.\n\n"));

 /* Set to zero for now */
 max_stages = 0;

 opt.cfg_lives        = DEFAULT_LIVES;
 opt.cfg_invincible   = DEFAULT_INVINCIBLE;
 opt.cfg_start_stage  = DEFAULT_STAGE;
 opt.cfg_showtext     = DEFAULT_SHOWTEXT;
 opt.cfg_sound        = DEFAULT_SOUND;
 opt.cfg_blockkind    = DEFAULT_BLOCKKIND;
 opt.cfg_enemykind    = DEFAULT_ENEMYKIND;
 opt.cfg_speed        = DEFAULT_SPEED;
 strcpy( opt.cfg_world_path, LINBERTO_WORLDFILE);
 opt.cfg_joystick     = DEFAULT_JOYSTICK;

 init_load_config();
 init_load_stage( opt.cfg_world_path);

 sound_init();

 menu_spacing  = DEFAULT_MENU_SPACING;
 menu_function = NULL;

 bright = 0;

 joy_init();
}

static void init_load_config( void)
{
 FILE *fp;
 struct config_file cfg;
 int i;

 check_if_file_exist( LINBERTO_CONFIGFILE);

 fp = linberto_openfile( LINBERTO_CONFIGFILE, LINBERTO_CONFIGFILE_DEFAULT, "r");

 i = fread( &cfg, 1, sizeof( struct config_file), fp);

 if( i > 0) {
     /* Prepare for older config file versions. */
     switch( i) {
             case 1068: opt.cfg_joystick    = cfg.cfg_joystick;
             case 1064: strcpy( opt.cfg_world_path, cfg.cfg_world_path);
             case 1056: opt.cfg_speed       = cfg.cfg_speed;
             case 32  : opt.cfg_enemykind   = cfg.cfg_enemykind;
             case 28  : opt.cfg_blockkind   = cfg.cfg_blockkind; 
             case 24  : opt.cfg_sound       = cfg.cfg_sound;
             case 20  : opt.cfg_start_stage = cfg.cfg_start_stage;
             case 16  : opt.cfg_invincible  = cfg.cfg_invincible;
             case 12  : opt.cfg_showtext    = cfg.cfg_showtext;
             case 8   : opt.cfg_lives       = cfg.cfg_lives;
             case 4   : opt.cfg_language    = cfg.cfg_language;
     }
 }

 nls_change_language();

 fclose( fp);
}

void init_save_config( void)
{
 FILE *fp;

 check_if_file_exist( LINBERTO_CONFIGFILE);

 fp = linberto_openfile( LINBERTO_CONFIGFILE, LINBERTO_CONFIGFILE_DEFAULT, "w");

 fwrite( &opt, sizeof( struct config_file), 1, fp);

 fclose( fp);
}

static void init_set_fonts( void)
{
 gl_setfont( 8, 8, linberto_font_8x8);

 gl_setwritemode( FONT_COMPRESSED + WRITEMODE_OVERWRITE);

 /* Background black, foreground white (with the game palette) */
 gl_setfontcolors( LIN_BLACK, LIN_WHITE);
}

void init_load_stage( char *filename)
{
 FILE *fp;
 char worldname[ 2048];
 struct world_header whdr;

 sprintf( worldname, "%s/%s", LINBERTO_DATADIR, filename);

 fp = linberto_openfile( worldname, LINBERTO_WORLDFILE_DEFAULT, "r");

 fread( &whdr, sizeof( struct world_header), 1, fp);

 if( strcmp( whdr.magic, WORLD_MAGIC)) {
     fprintf( stderr, _("\nThe file [%s] isn't a Linberto world file.\n\n"), openedfile);
     exit_game( 1);
 }

 max_stages = whdr.number;

 fclose( fp);
}

